package org.bitbucket.bradleysmithllc.etlunit;

import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestClass;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestParser;
import org.bitbucket.bradleysmithllc.etlunit.regexp.PackNameExpression;
import org.bitbucket.bradleysmithllc.etlunit.util.NeedsTest;

import javax.inject.Inject;
import javax.inject.Named;
import java.io.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Stack;
import java.util.regex.Pattern;

@NeedsTest
public class DirectoryBasedClassLocatorImpl implements ClassLocator
{
	private final File rootDir;
	private Log applicationLog;

	private final List<ETLTestClass> classesList = new ArrayList<ETLTestClass>();
	private int currentClassPointer = 0;

	private final Stack<File> dirStack = new Stack<File>();
	private final Stack<ETLTestClass> classStack = new Stack<ETLTestClass>();

	public DirectoryBasedClassLocatorImpl(File dir)
	{
		rootDir = dir;

		if (!rootDir.exists())
		{
			throw new IllegalArgumentException("Root directory " + rootDir + " does not exist");
		}

		// the implementation has changed.  It used to be a pass-through
		// implementation, but now it caches all classes and sorts after the fact
	}

	@Inject
	void setLogger(@Named("applicationLog") Log log)
	{
		applicationLog = log;
	}

	public boolean hasNext0()
	{
		if (classStack.size() != 0)
		{
			return true;
		}

		if (dirStack.size() == 0)
		{
			return false;
		}

		// at this point, the class stack is empty and the dir stack is not.
		// now pop the next dir and process until either the dir stack is
		// depleted or we have at least one class to process
		while (classStack.size() == 0)
		{
			if (dirStack.size() == 0)
			{
				return false;
			}

			File dir = dirStack.pop();

			// try to determine the package name
			String
					_package =
					dir.getAbsolutePath().substring(rootDir.getAbsolutePath().length());

			if (_package.startsWith(File.separator))
			{
				_package = _package.substring(File.separator.length());
			}

			if (_package.endsWith(File.separator))
			{
				_package = _package.substring(_package.length() - File.separator.length());
			}

			final String __package = smoothPackage(_package);

			// scan the dir.  Every .etlunit class is loaded, parsed, and added to the
			// classes stack.  Every directory is pushed onto the stack.  This will have the
			// affect of a depth-first traversal of the directory tree
			dir.listFiles(new FileFilter()
			{
				@Override
				public boolean accept(File pathname)
				{
					if (pathname.isDirectory())
					{
						PackNameExpression pne = new PackNameExpression(pathname.getName());

						if (pne.hasNext())
						{
							dirStack.push(pathname);
						}
					}
					else if (pathname.isFile() && pathname.getName().toLowerCase().endsWith(".etlunit"))
					{
						// parse the class file and add to the stack
						try
						{
							Reader fileReader = new BufferedReader(new FileReader(pathname));

							try
							{
								List<ETLTestClass> list = ETLTestParser.load(fileReader, __package);

								classStack.addAll(list);
							}
							finally
							{
								fileReader.close();
							}
						}
						catch (Exception e)
						{
							// log the error somewhere
							applicationLog.severe("", e);
						}
					}

					return false;
				}
			});
		}

		return true;
	}

	private String smoothPackage(String aPackage)
	{
		if (aPackage.length() == 0)
		{
			return aPackage;
		}

		StringBuilder stb = new StringBuilder();

		String [] packs = aPackage.split(Pattern.quote(File.separator));

		for (String pn : packs)
		{
			PackNameExpression pne = new PackNameExpression(pn);

			if (pne.matches())
			{
				if (stb.length() != 0)
				{
					stb.append(".");
				}

				stb.append(pne.getPackName());
			}
			else
			{
				throw new RuntimeException("Horrible package name " + pn);
			}
		}

		return stb.toString();
	}

	public ETLTestClass next0()
	{
		return classStack.pop();
	}

	@Override
	public boolean hasNext()
	{
		if (classesList.size() == 0)
		{
			//seed the stack with the root dir
			dirStack.clear();
			classStack.clear();

			dirStack.push(rootDir);

			while (hasNext0())
			{
				ETLTestClass class_ = next0();

				classesList.add(class_);
			}

			Collections.sort(classesList);
		}

		return currentClassPointer < classesList.size();
	}

	@Override
	public ETLTestClass next()
	{
		return classesList.get(currentClassPointer++);
	}

	@Override
	public void remove()
	{
		throw new UnsupportedOperationException();
	}

	@Override
	public void reset()
	{
		// just reset the current class pointer to 0;
		currentClassPointer = 0;
	}
}