package org.bitbucket.bradleysmithllc.etlunit.metadata.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.google.gson.stream.JsonWriter;
import org.apache.commons.lang3.ObjectUtils;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.metadata.MetaDataContext;
import org.bitbucket.bradleysmithllc.etlunit.metadata.MetaDataPackageContext;

import javax.inject.Inject;
import java.io.IOException;
import java.util.*;

public class MetaDataContextImpl implements MetaDataContext
{
	private RuntimeSupport runtimeSupport;

	private String name;
	private final List<MetaDataPackageContext> contexts	= new ArrayList<MetaDataPackageContext>();
	private final Map<String, MetaDataPackageContext> packageContextMap = new HashMap<String, MetaDataPackageContext>();

	public MetaDataContextImpl(String name) {
		this.name = name;
	}

	public MetaDataContextImpl() {
	}

	@Inject
	public void receiveRuntimeSupport(RuntimeSupport support)
	{
		runtimeSupport = support;
	}

	@Override
	public String getContextName() {
		return name;
	}

	@Override
	public Map<String, MetaDataPackageContext> getPackageContextMap() {
		return Collections.unmodifiableMap(packageContextMap);
	}

	@Override
	public MetaDataPackageContext createPackageContext(String pack, MetaDataPackageContext.path_type pathType) {
		String key = key(pack, pathType);

		if (packageContextMap.containsKey(key))
		{
			return packageContextMap.get(key);
		}

		MetaDataPackageContextImpl mpci = new MetaDataPackageContextImpl(pack, pathType);
		mpci.receiveRuntimeSupport(runtimeSupport);

		packageContextMap.put(key, mpci);
		contexts.add(mpci);

		return mpci;
	}

	@Override
	public MetaDataPackageContext createPackageContextForCurrentTest(MetaDataPackageContext.path_type pathType) {
		if (runtimeSupport == null)
		{
			throw new IllegalStateException("Runtime support not available");
		}

		return createPackageContext(runtimeSupport.getCurrentlyProcessingTestPackage(), pathType);
	}

	@Override
	public List<MetaDataPackageContext> getPackageContexts() {
		return Collections.unmodifiableList(contexts);
	}

	@Override
	public void toJson(JsonWriter writer) throws IOException {
		writer.name("name").value(name);

		writer.name("package-contexts");
		writer.beginArray();

		for (MetaDataPackageContext mpci : contexts)
		{
			writer.beginObject();
			mpci.toJson(writer);
			writer.endObject();
		}

		writer.endArray();
	}

	@Override
	public void fromJson(JsonNode node) {
		contexts.clear();
		packageContextMap.clear();

		name = node.get("name").asText();

		ArrayNode anode = (ArrayNode) node.get("package-contexts");

		Iterator<JsonNode> it = anode.elements();

		while (it.hasNext())
		{
			MetaDataPackageContextImpl mpci = new MetaDataPackageContextImpl();
			mpci.receiveRuntimeSupport(runtimeSupport);

			mpci.fromJson(it.next());
			String key = key(mpci.getPackageName(), mpci.getPathType());

			contexts.add(mpci);
			packageContextMap.put(key, mpci);
		}
	}

	private String key(String packageName, MetaDataPackageContext.path_type pathType) {
		return (packageName == null ? "" : "[" + packageName + "]") + "[" + pathType + "]";
	}

	public boolean equals(Object other)
	{
		if (getClass() != other.getClass())
		{
			return false;
		}

		MetaDataContextImpl mpci = (MetaDataContextImpl) other;

		return ObjectUtils.compare(name, mpci.name) == 0 && contexts.equals(mpci.contexts);
	}
}